require 'kaminari'
require 'active_model_serializers'

module PaginatedSerializer
  class Serializer
    attr_accessor :collection

    def initialize(collection)
      @collection = collection
    end

    def to_json
      {
        per_page: per_page,
        total: total,
        total_pages: total_pages,
        last_page: last_page,
        current_page: current_page,
        first_page: first_page,
        next_page: next_page,
        previous_page: previous_page
      }.merge(collection_json)
    end

    private

    def per_page
      Kaminari.config.default_per_page
    end

    def total
      collection.total_count
    end

    def total_pages
      collection.total_pages
    end

    def last_page
      collection.total_pages
    end

    def current_page
      collection.current_page
    end

    def first_page
      1
    end

    def next_page
      if total_pages > current_page
        collection.current_page + 1
      end
    end

    def previous_page
      if current_page > 1
        collection.current_page - 1
      end
    end

    def collection_json
      klass = collection.first.class.name
      key = klass.pluralize.underscore.to_sym
      serializer = "#{klass}Serializer".constantize
      { key => ActiveModel::ArraySerializer.new(collection, each_serializer: serializer).as_json }
    end
  end
end
